""" Etude de la charge  du circuit RC avec fichier de données en format txt """

import matplotlib.pyplot as plt
from matplotlib.widgets import Cursor
import numpy as np

""" remarque: on peut aussi importer la bibliothèque pylab qui inclut matplotlib et numpy
et qui a une syntaxe proche de Mathlab avec écriture plus naturelle, par exemple:
exp(x) au lieu de np.exp(x)
from pylab import * pour inclure toutes les fonctions de pylab"""
import math
import os # os: operating system: module permettant de dialoguer avec le système d'exploitation

R = float(input("Entrer la valeur de R (en ohm) :   "))
C = float(input("Entrer la valeur de C (en farad):    "))
nomFichier = str(input('Nom du fichier à lire '))
fichier_de_mesures = nomFichier+ '.txt'
#exemple : NomFichier ='10k-47m-charge.txt'
repertoire=str(input('Nom du repertoire de travail:  '))
os.chdir(repertoire) # définit le répertoire de travail
# ou bien, écrire par exemple: os.chdir("E:/Python/exercices")
# cree le chemin d'acces au ficher XXX.txt
# chdir: change directory

#***************************************************************************
# Lecture des données du fichier et création de tableaux de valeurs
# ***************************************************************************

def lecture(fichier):
    f = open(fichier)
    lignes  = f.readlines() #transfere toutes les lignes dans une liste de chaînes
    lignes  = lignes[2:] # la lecture des données numériques se fait seulement à partir de la troisième ligne
    temps,E,Uc = [],[],[]
    for line in lignes: # boucle iterative de lecture des lignes du fichier
        t,e,uc = line.split('\t')# split('\t') le caractère tabulation n'est pas lu
        temps.append(float(t)) # ajout de la valeur de t au tableau 1D temps
        E.append(float(e))
        Uc.append(float(uc))
    f.close() # toujours fermer le fichier après ouverture
    return temps,E,Uc

# On attribue les noms aux 3 listes contenues dans le fichier
t1,u0,u1=(lecture(fichier_de_mesures))

# **********************************************************************************
# calculs des temps caractéristiques et des valeurs modélisées des tensions uc et uR
#***********************************************************************************

# détermination de tau et Ucmax
def modele(t1,u):
    umax = max(u)
    for i in range(len(t1)-1):# lecture jusqu'à la dernière mesure
        if u[i]<=umax*0.63 and u[i+1]>=umax*0.63:
            tauexp=(t1[i]+t1[i+1])/2
            return umax,tauexp

Umax,tauexp=modele(t1,u1)
print("Umax  :\t",Umax," V")
print("tau experimental :\t",round(tauexp,2), "s")
# on arrondit à deux décimales

# calcul de tau theorique
def calculTau(R,C):
    tau =R*C
    return tau

tau=calculTau(R,C)
print("tau theorique : ",round(tau,2)," s")

# Modele
def ucm(t):
    return Umax*(1-np.exp(- t/tauexp))

Ucm=[]
for i in range(len(t1)):
    Ucm.append(ucm(t1[i]))

# Calcul de Ur
Ur=[]
for i in range(len(t1)):
    Ur.append(Umax-ucm(t1[i]))

# ***************************************************************************
# Tracé des graphes: voir https://courspython.com/introduction-courbes.html
# ***************************************************************************
# Les 3 graphes
fig1 = plt.subplot(221)
plt.plot(t1,u0,"b*",label='e(t)')
plt.plot(t1,u1,"r+",label='uc(t)')
plt.xlabel("t (s)")
plt.ylabel("tension (V)")
plt.axis([0,t1[-1],0,5.5])#plt.axis([xmin,xmax,ymin,ymax])
plt.title('uc(t) et e(t)')
plt.legend(loc='best')
plt.grid()
cursor1 = Cursor(fig1, useblit=True, color='blue', linewidth=1)
# Tracé temporel du signal et modelisation
fig2 = plt.subplot(222)
plt.plot(t1,u1,"r+",label='uc(t)')
plt.plot(t1,Ucm,'b',label='ucm(t)')
plt.xlabel("t (s)")
plt.ylabel("tension (V)")
plt.axis([0,t1[-1],0,5.5])
plt.title('uc(t) et son modele')
plt.legend(loc='best')
plt.grid()
cursor2 = Cursor(fig2, useblit=True, color='green', linewidth=1)
# Tracé temporel du signal et de ur(t)
fig3 = plt.subplot(223)
plt.plot(t1,u1,"r+",label='uc(t)')
plt.plot(t1,Ur,'b',label='ur(t)')
plt.xlabel("t (s)")
plt.ylabel("tension (V)")
plt.legend(loc='best')
plt.title('uc(t) et ur(t)')
plt.grid()
cursor3 = Cursor(fig3, useblit=True, color='black', linewidth=1)
plt.show()

# sauvegarde les graphes dans le fichier image au format png dans
#le répertoire défini par l'opérateur avec un nom identique au nom du fichier de mesures
image = nomFichier
plt.savefig(nomFichier)

